package com.liaobo.im.ui.me;

import android.content.Intent;
import android.os.Bundle;
import android.text.TextUtils;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.liaobo.im.AppConstant;
import com.liaobo.im.R;
import com.liaobo.im.db.dao.UserDao;
import com.liaobo.im.helper.AvatarHelper;
import com.liaobo.im.helper.DialogHelper;
import com.liaobo.im.ui.base.BaseActivity;
import com.liaobo.im.ui.base.CoreManager;
import com.liaobo.im.util.SkinUtils;
import com.liaobo.im.util.ToastUtil;
import com.liaobo.im.xuan.xuanhttplibrary.okhttp.HttpUtils;
import com.liaobo.im.xuan.xuanhttplibrary.okhttp.callback.BaseCallback;
import com.liaobo.im.xuan.xuanhttplibrary.okhttp.result.ObjectResult;

import java.util.HashMap;
import java.util.Map;

import okhttp3.Call;

/**
 * 设置app_name号
 */
public class SetAccountActivity extends BaseActivity {
    private String mUserId, mUserName;
    private ImageView mAccountAvatarIv;
    private TextView mAccountNameTv;
    private EditText mAccountInputEt;
    private Button mAccountSureBtn;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_set_account);
        mUserId = getIntent().getStringExtra(AppConstant.EXTRA_USER_ID);
        mUserName = getIntent().getStringExtra(AppConstant.EXTRA_NICK_NAME);
        initActionBar();
        initView();
        initData();
    }

    private void initActionBar() {
        getSupportActionBar().hide();
        findViewById(R.id.iv_title_left).setOnClickListener(v -> finish());
        TextView tvTitle = findViewById(R.id.tv_title_center);
        tvTitle.setText(getString(R.string.sk_account_set, getString(R.string.sk_account_code)));
    }

    private void initView() {
        TextView singleDesc = findViewById(R.id.sk_account_single_tv);
        singleDesc.setText(getString(R.string.sk_account_single, getString(R.string.sk_account_code)));
        mAccountAvatarIv = findViewById(R.id.a_avatar_iv);
        mAccountNameTv = findViewById(R.id.a_name_tv);
        mAccountInputEt = findViewById(R.id.a_input_et);
        mAccountSureBtn = findViewById(R.id.a_sure_btn);
        mAccountSureBtn.setBackgroundColor(SkinUtils.getSkin(this).getAccentColor());

    }

    private void initData() {
        AvatarHelper.getInstance().displayAvatar(mUserId, mAccountAvatarIv, true);
        mAccountNameTv.setText(mUserName);
        mAccountSureBtn.setOnClickListener(v -> {
            String account = mAccountInputEt.getText().toString();
            if (TextUtils.isEmpty(account)) {
                Toast.makeText(mContext, getString(R.string.name_connot_null), Toast.LENGTH_SHORT).show();
                return;
            }
            updateAccount(account);
        });
    }

    private void updateAccount(String account) {
        DialogHelper.showDefaulteMessageProgressDialog(this);
        Map<String, String> params = new HashMap<>();
        params.put("access_token", CoreManager.getSelfStatus(mContext).accessToken);
        params.put("account", account);

        HttpUtils.get().url(coreManager.getConfig().USER_UPDATE)
                .params(params)
                .build()
                .execute(new BaseCallback<Void>(Void.class) {

                    @Override
                    public void onResponse(ObjectResult<Void> result) {
                        DialogHelper.dismissProgressDialog();
                        if (result.getResultCode() == 1) {
                            coreManager.getSelf().setAccount(account);
                            coreManager.getSelf().setSetAccountCount(1);
                            // 更新数据库
                            UserDao.getInstance().updateAccount(mUserId, account);

                            Intent intent = new Intent();
                            intent.putExtra(AppConstant.EXTRA_USER_ACCOUNT, account);
                            setResult(RESULT_OK, intent);
                            finish();
                        } else {
                            Toast.makeText(SetAccountActivity.this, result.getResultMsg(), Toast.LENGTH_SHORT).show();
                        }
                    }

                    @Override
                    public void onError(Call call, Exception e) {
                        DialogHelper.dismissProgressDialog();
                        ToastUtil.showErrorNet(SetAccountActivity.this);
                    }
                });
    }
}
