package com.liaobo.im.ui.me.redpacket;

import android.os.Bundle;
import android.support.annotation.NonNull;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ListView;
import android.widget.TextView;

import com.liaobo.im.R;
import com.liaobo.im.bean.redpacket.BaseRedPacketItemEntity;
import com.liaobo.im.bean.redpacket.RedPacketReceiveBean;
import com.liaobo.im.bean.redpacket.RedPacketSendBean;
import com.liaobo.im.bean.redpacket.RedSendRedItemEntry;
import com.liaobo.im.db.InternationalizationHelper;
import com.liaobo.im.helper.DialogHelper;
import com.liaobo.im.ui.base.BaseActivity;
import com.liaobo.im.xuan.xuanhttplibrary.okhttp.HttpUtils;
import com.liaobo.im.xuan.xuanhttplibrary.okhttp.callback.BaseCallback;
import com.liaobo.im.xuan.xuanhttplibrary.okhttp.result.ObjectResult;
import com.scwang.smartrefresh.layout.SmartRefreshLayout;
import com.scwang.smartrefresh.layout.api.RefreshLayout;
import com.scwang.smartrefresh.layout.listener.OnRefreshLoadMoreListener;

import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

import okhttp3.Call;

/**
 * 红包记录界面Activity
 * Created by 魏正旺 on 2016/9/23.
 */
public class RedListActivity extends BaseActivity {

    // 数据格式，保留两位小数
    DecimalFormat df;
    // 显示时间的格式
    SimpleDateFormat sdf;

    public static final String ADAPTER_TYPE = "adapterType";
    // 数据集合
//    private RedPacketSendBean redSendBean;
//    private RedPacketReceiveBean mRedPacketReceviceBean;
    // 表示发出的还是接受的红包list  0代表收到  1代表发出 2代表专属红包
    private int adapterType;
    // 两个listview数据加载的页数
    private int mPageIndex = 0;
    private RedListItemAdapter redListItemAdapter;
    private SmartRefreshLayout smartRefreshLayout;
    private ListView mRedPacketListView;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.redpacket_list);

        getSupportActionBar().hide();
        findViewById(R.id.iv_title_left).setOnClickListener(view -> finish());
        TextView tvTitle = findViewById(R.id.tv_title_center);
        adapterType = getIntent().getIntExtra(ADAPTER_TYPE, 0);
        if (adapterType == 0) {
            redListItemAdapter = new RedListItemAdapter<>();
            tvTitle.setText(InternationalizationHelper.getString("PACKETS_RECEIVED"));

        } else if (adapterType == 1) {
            redListItemAdapter = new RedListItemAdapter<RedSendRedItemEntry>();
            tvTitle.setText(InternationalizationHelper.getString("ENVELOPES_ISSUED"));
        } else {
            tvTitle.setText(R.string.view_auto_get_red_packet_title);
            redListItemAdapter = new RedListItemAdapter<>();
        }
        mRedPacketListView = findViewById(R.id.red_packet_list_view);
        smartRefreshLayout = findViewById(R.id.smart_refresh_layout);

        df = new DecimalFormat("######0.00");
        sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm");

        initView();
        getData(mPageIndex);
    }

    private void initView() {

//        redSendBean = new RedPacketSendBean();
//        mRedPacketReceviceBean = new RedPacketReceiveBean();

        // 设置Adapter
        redListItemAdapter = new RedListItemAdapter();
        mRedPacketListView.setAdapter(redListItemAdapter);
        smartRefreshLayout.setOnRefreshLoadMoreListener(new OnRefreshLoadMoreListener() {
            @Override
            public void onLoadMore(@NonNull RefreshLayout refreshLayout) {
                getData(mPageIndex + 1);
            }

            @Override
            public void onRefresh(@NonNull RefreshLayout refreshLayout) {
                smartRefreshLayout.setEnableLoadMore(true);
                mPageIndex = 0;
                getData(mPageIndex);

            }
        });

    }


    private void getData(int pageIndex) {
        if (adapterType == 0) {
            loadReceiveRedPacket(pageIndex);
        } else if (adapterType == 1) {
            loadSendRed(pageIndex);
        } else {
            loadAutoGetAssignRedReceiveList(pageIndex);
        }
    }

    private void loadSendRed(int pageIndex) {
        HashMap<String, String> params = new HashMap<String, String>();
        params.put("access_token", coreManager.getSelfStatus().accessToken);
        params.put("pageIndex", pageIndex + "");
        params.put("pageSize", "25");
        params.put("secret", "");

        HttpUtils.get().url(coreManager.getConfig().SEND_REDPACKET_LIST_GET)
                .params(params)
                .build()
                .execute(new BaseCallback<RedPacketSendBean>(RedPacketSendBean.class) {

                    @Override
                    public void onResponse(ObjectResult<RedPacketSendBean> result) {
                        smartRefreshLayout.finishLoadMore();
                        smartRefreshLayout.finishRefresh();
                        if (result != null && result.getData() != null && result.getData().getList() != null && result.getData().getList().size() > 0) {
                            List<RedSendRedItemEntry> dataList = result.getData().getList();

                            if (mPageIndex == 0) {
                                redListItemAdapter.setData(dataList);

                            } else {
                                redListItemAdapter.addData(dataList);
                            }
                            redListItemAdapter.notifyDataSetChanged();
                            mPageIndex++;
                        } else {
                            smartRefreshLayout.setEnableLoadMore(false);
                        }

                    }

                    @Override
                    public void onError(Call call, Exception e) {
                        smartRefreshLayout.finishLoadMore();
                        smartRefreshLayout.finishRefresh();
                    }
                });
    }

    private void loadAutoGetAssignRedReceiveList(int pageIndex) {
        DialogHelper.showMessageProgressDialog(this,getString(R.string.please_wait));
        HashMap<String, String> params = new HashMap<String, String>();
        params.put("access_token", coreManager.getSelfStatus().accessToken);
        params.put("pageIndex", pageIndex + "");
        params.put("pageSize", 25 + "");

        HttpUtils.get().url(coreManager.getConfig().GET_AUTO_ASSIGN_RED_RECEIVE_LIST)
                .params(params)
                .build()
                .execute(new BaseCallback<RedPacketReceiveBean>(RedPacketReceiveBean.class) {

                    @Override
                    public void onResponse(ObjectResult<RedPacketReceiveBean> result) {
                        DialogHelper.dismissProgressDialog();
                        smartRefreshLayout.finishLoadMore();
                        smartRefreshLayout.finishRefresh();

                        if (result != null && result.getData() != null && result.getData().list != null && result.getData().list.size() > 0) {
                            List<BaseRedPacketItemEntity> dataList = result.getData().list;
                            if (mPageIndex == 0) {
                                redListItemAdapter.setData(dataList);
                            } else {
                                redListItemAdapter.addData(dataList);
                            }
                            mPageIndex++;
                        } else {
                            smartRefreshLayout.setEnableLoadMore(false);
                        }

                    }

                    @Override
                    public void onError(Call call, Exception e) {
                        DialogHelper.dismissProgressDialog();
                        smartRefreshLayout.finishLoadMore();
                        smartRefreshLayout.finishRefresh();
                    }
                });
    }


    private void loadReceiveRedPacket(int pageIndex) {
        HashMap<String, String> params = new HashMap<String, String>();
        params.put("access_token", coreManager.getSelfStatus().accessToken);
        params.put("pageIndex", pageIndex + "");
        params.put("pageSize", 25 + "");

        HttpUtils.get().url(coreManager.getConfig().RECIVE_REDPACKET_LIST_GET)
                .params(params)
                .build()
                .execute(new BaseCallback<RedPacketReceiveBean>(RedPacketReceiveBean.class) {

                    @Override
                    public void onResponse(ObjectResult<RedPacketReceiveBean> result) {
                        smartRefreshLayout.finishLoadMore();
                        smartRefreshLayout.finishRefresh();

                        if (result != null && result.getData() != null && result.getData().list != null && result.getData().list.size() > 0) {
                            List<BaseRedPacketItemEntity> dataList = result.getData().list;
                            if (mPageIndex == 0) {
                                redListItemAdapter.setData(dataList);
                            } else {
                                redListItemAdapter.addData(dataList);
                            }
                            mPageIndex++;
                        } else {
                            smartRefreshLayout.setEnableLoadMore(false);
                        }

                    }

                    @Override
                    public void onError(Call call, Exception e) {
                        smartRefreshLayout.finishLoadMore();
                        smartRefreshLayout.finishRefresh();
                    }
                });
    }


    /**
     * listview Adapter
     */
    public class RedListItemAdapter<T extends BaseRedPacketItemEntity> extends BaseAdapter {
        List<T> mList = new ArrayList<>();

        @Override
        public int getCount() {
            return mList == null ? 0 : mList.size();
        }

        @Override
        public Object getItem(int position) {
            return null;
        }

        public void addData(List<T> list) {
            if (list != null) {
                mList.addAll(list);
                notifyDataSetChanged();
            }
        }

        public void addData(T entity) {
            mList.add(entity);
            notifyDataSetChanged();
        }

        public void setData(List<T> list) {
            if (list != null) {
                mList = list;
                notifyDataSetChanged();
            }
        }

        @Override
        public long getItemId(int position) {
            return 0;
        }

        @Override
        public View getView(int position, View convertView, ViewGroup parent) {

            RedViewHolder redViewHolder;
            if (convertView == null) {
                LayoutInflater inflater = LayoutInflater.from(getApplicationContext());
                convertView = inflater.inflate(R.layout.red_item, null);
                redViewHolder = new RedViewHolder();

                redViewHolder.userNameTv = (TextView) convertView.findViewById(R.id.username_tv);
                redViewHolder.moneyTv = (TextView) convertView.findViewById(R.id.money_tv);
                redViewHolder.timeTv = (TextView) convertView.findViewById(R.id.time_tv);
                convertView.setTag(redViewHolder);
            } else {
                redViewHolder = (RedViewHolder) convertView.getTag();
            }

            if (adapterType == 0 || adapterType == 2) {
                long lcc_time = mList.get(position).time;
                String StrTime = sdf.format(new Date(lcc_time * 1000L));
                redViewHolder.userNameTv.setText(mList.get(position).sendName);
                redViewHolder.timeTv.setText(StrTime);
                redViewHolder.moneyTv.setText(df.format(mList.get(position).money) + InternationalizationHelper.getString("YUAN"));
            } else if (adapterType == 1) {
                int type = ((RedSendRedItemEntry) mList.get(position)).type;
                switch (type) {


                    case 2: {
                        //                        redViewHolder.userNameTv.setText("拼手气红包");
                        redViewHolder.userNameTv.setText(InternationalizationHelper.getString("JX_LuckGift"));
                    }
                    break;

                    case 3: {
                        //                        redViewHolder.userNameTv.setText("口令红包");
                        redViewHolder.userNameTv.setText(InternationalizationHelper.getString("JX_MesGift"));
                    }
                    break;
                    case 4:
                        redViewHolder.userNameTv.setText(R.string.exclusive_red_packet1);
                    break;
                    case 1:
                    default:
                        //                        redViewHolder.userNameTv.setText("普通红包");
                        redViewHolder.userNameTv.setText(InternationalizationHelper.getString("JX_UsualGift"));

                    break;

                }
                long lcc_time = ((RedSendRedItemEntry) mList.get(position)).sendTime;
                String StrTime = sdf.format(new Date(lcc_time * 1000L));
                redViewHolder.timeTv.setText(StrTime);
                redViewHolder.moneyTv.setText(df.format(mList.get(position).money) + InternationalizationHelper.getString("YUAN"));
            }
            return convertView;
        }

        public class RedViewHolder {
            public TextView userNameTv;
            public TextView timeTv;
            public TextView moneyTv;
        }
    }
}
